import type { Context, Handler } from 'hono';
import { type WebSocketConnection } from './handlers/websocket';
export declare const createWorkbenchExecutionRoute: () => Handler;
export declare const createWorkbenchClearStateRoute: () => Handler;
export declare const createWorkbenchStateRoute: () => Handler;
/**
 * Creates a workbench router with proper agent middleware for execution routes
 */
export declare const createWorkbenchRouter: () => import("hono").Hono<import("./app").Env<Record<string, never>>, import("hono/types").BlankSchema, "/">;
export declare const createWorkbenchSampleRoute: () => Handler;
export declare const createWorkbenchMetadataRoute: () => Handler;
export declare const createWorkbenchWebsocketHandler: () => (_c: Context, ws: WebSocketConnection) => void;
/**
 * @deprecated Use createWorkbenchWebsocketHandler instead
 */
export declare const createWorkbenchWebsocketRoute: () => (_c: Context, ws: WebSocketConnection) => void;
//# sourceMappingURL=workbench.d.ts.map