import { Hono } from 'hono';
/**
 * Creates a Hono router for use with Agentuity.
 *
 * This is a thin wrapper around `new Hono()` that provides the correct
 * Agentuity environment types. Hono's full type inference chain is
 * preserved — the Schema type parameter accumulates route definitions
 * as you chain `.get()`, `.post()`, etc.
 *
 * @template E - Environment type (defaults to Agentuity's Env)
 * @template S - Schema type for route definitions
 *
 * @returns Hono router instance
 *
 * @example
 * ```typescript
 * import { createRouter, websocket, sse, stream, cron } from '@agentuity/runtime';
 *
 * const router = createRouter();
 *
 * // Standard HTTP routes — full type inference
 * router.get('/hello', (c) => c.text('Hello!'));
 * router.post('/data', async (c) => {
 *   const body = await c.req.json();
 *   return c.json({ received: body });
 * });
 *
 * // WebSocket connection
 * router.get('/ws', websocket((c, ws) => {
 *   ws.onMessage((event) => {
 *     ws.send('Echo: ' + event.data);
 *   });
 * }));
 *
 * // Server-Sent Events
 * router.get('/events', sse((c, stream) => {
 *   stream.writeSSE({ data: 'Hello', event: 'message' });
 * }));
 * ```
 */
export const createRouter = () => {
    return new Hono();
};
//# sourceMappingURL=router.js.map