import type { Database } from 'bun:sqlite';
import type { QueueService, QueuePublishParams, QueuePublishResult, QueueCreateParams, QueueCreateResult } from '@agentuity/core';
export declare class LocalQueueStorage implements QueueService {
    #private;
    constructor(db: Database, projectPath: string);
    publish(queueName: string, payload: string | object, params?: QueuePublishParams): Promise<QueuePublishResult>;
    createQueue(queueName: string, params?: QueueCreateParams): Promise<QueueCreateResult>;
    deleteQueue(_queueName: string): Promise<void>;
}
//# sourceMappingURL=queue.d.ts.map