import { Hono } from 'hono';
/**
 * Create a router that serves the web application.
 * In dev mode (DEV=true), serves HTML with Vite HMR scripts (Bun server proxies assets to Vite).
 * In production, serves static files from .agentuity/client/.
 */
export declare function createWebRouter(): Promise<Hono>;
//# sourceMappingURL=web.d.ts.map