import { createLogger } from '@agentuity/server';
/**
 * User-facing logger instance
 * This is the logger that SDK consumers should use
 */
export const logger = createLogger((process.env.AGENTUITY_LOG_LEVEL || 'info'), false, (process.env.COLOR_SCHEME ?? 'dark'));
//# sourceMappingURL=user.js.map