import type { Logger } from '@agentuity/core';
/**
 * User-facing logger instance
 * This is the logger that SDK consumers should use
 */
export declare const logger: Logger;
export type { Logger } from '@agentuity/core';
//# sourceMappingURL=user.d.ts.map